C$Procedure                         Init_parms

      subroutine init_parms

C
C*******************************************************************************
C
C       Copyright (C) 1993, California Institute of Technology.  U.S.
C       Government Sponsorhip under NASA Contract NAS7-918 is
C       acknowledged.
C
C*******************************************************************************
C
C
C$ Log
C
C  Date         Name               Description
C  -----------------------------------------------------------------------------
C  30-Sep-1991  B. Shapiro	   creation
C
C$ Purpose
C
C  Initializes the derived physical constants (e.g., earth rotation 
C  rate).
C
C$ Input_Arguments
C
C  Name    Type   Dim        Units   Description
C  -----------------------------------------------------------------------------
C	none
C
C$ Output_Arguments
C
C  Name    Type   Dim        Units   Description
C  -----------------------------------------------------------------------------
C	none
C
C$ Namelist_Output
C
C  Name    Type   Dim        Units   Description
C  -----------------------------------------------------------------------------
C  	none
C
C
C$ Parameters
C
      double precision     pi
      PARAMETER          ( PI     = 3. 14159 26535 89793 23846 D0 )
C
C$ Declarations_of_Global_Variables
C
C  Name    Type   Dim        Units   Description
C  -----------------------------------------------------------------------------
C  earth_rad d    1	     km	          earth equatorial radius
C  earth_freq d   1	     rad/sec      earth rotation frequency
C  earth_rate d   1          meters/day   earth rotation speed
C  mu_earth   d   1          km**3/sec**2 Earth gravity constant
C  mu_moon    d   1          km**3/sec**2 Earth gravity constant
C  mu_sun     d   1          km**3/sec**2 Earth gravity constant
C  sid_day    d   1          seconds      Length of sidereal day 
C  deg_to_km  d   1          km/degree    1 degree of longitude at equator
C

      double precision earth_rad		! in kilomters
      double precision earth_freq 		! radians / second
      double precision earth_rate		! kilomters / day
      double precision mu_earth			! km**3/sec
      double precision mu_moon			! km**3/sec
      double precision mu_sun			! km**3/sec
      double precision sid_day			! seconds
      double precision deg_to_km	 	! kilometers/deg

      common / physical_constants / 
     &	  earth_rad,  earth_freq,  earth_rate,    mu_earth,        
     &    mu_moon,    mu_sun,         sid_day,    deg_to_km

C$ External_Statements
C
C	none
C
C$ Method
C
      deg_to_km = pi * earth_rad /  180.0d0
    
      earth_rate = 2 * pi * earth_rad * 86400000.0 / sid_day 

      earth_freq =  2 * pi / sid_day
      
      return
      end

