C$Procedure                         LSRGPB
C
      SUBROUTINE LSRGPB ( IOFLAG , ITFLAG , ITVALU )
C
C
C*******************************************************************************
C
C       Copyright (C) 1993, California Institute of Technology.  U.S.
C       Government Sponsorhip under NASA Contract NAS7-918 is
C       acknowledged.
C
C*******************************************************************************
C
C$ Log
C
C  19-JUN-1990 - Eric Cannell     - creation of LSRGPB
C
C$ Purpose
C
C  LSRGPB is the bulletin board for the LSRGP library. Rather than using
C  common blocks to pass data out of the LSRGP library, LSRGP will simply post 
C  a value on the LSRGP bulletin board. A calling program, such as EQXING, 
C  can then read the board for the posted value.
C
C  ADDING SLOTS TO THE BULLETIN BOARD:
C
C    A programmer can add more slots to the bulletin board by creating
C    three additional variables and adding two ELSE IF clauses.
C
C    The first variable is the xID parameter used to identify the slot,
C    the second is the SAVEd xVAL variable that holds the actual value,
C    and the third is the SAVEd xSET variable that makes sure that a 
C    value has been "PUT" before it can be "GET"ed. Be sure to DATA set 
C    the xSET variable.
C
C    One of the ELSE IF clauses is for PUTting and the other for GETting.
C
C  Currently, the only values that can be posted and read are:
C              
C        item   |   ITFLAG   | description
C        -------|------------|----------------------------------------------
C         N     | "LSRGP_N"  | the mean motion rads/sec
C         N     | "LSRGP_NHI"  | the mean motion rads/sec
C         N     | "LSRGP_NLO"  | the mean motion rads/sec
C
C$ Input_Arguments
C
C  Name    Type   Dim        Units   Description
C  -----------------------------------------------------------------------------
C  IOFLAG   C*3     1            -   either "PUT"s or "GET"s a value from
C                                    the bulletin board
C  ITFLAG C*(*)     1            -   string which identifies value, see purpose
C  ITVALU    DP     1            ?   value to be posted
C
C$ Output_Arguments
C
C  Name    Type   Dim        Units   Description
C  -----------------------------------------------------------------------------
C  ITVALU    DP     1            ?   value to be gotten
C
C$ Restrictions
C
C  1] LSRGPB will only handle double precision values.
C
C$ Library_Links
C
C  TPXUTIL              
C
C$ Parameters
C
      CHARACTER*3          GET
      PARAMETER          ( GET    = 'GET' )

      CHARACTER*3          PUT
      PARAMETER          ( PUT    = 'PUT' )

      CHARACTER*7          NID
      PARAMETER          ( NID    = 'LSRGP_N' )

      CHARACTER*7          NIDHI
      PARAMETER          ( NIDHI    = 'LSRGP_NHI' )

      CHARACTER*7          NIDLO
      PARAMETER          ( NIDLO    = 'LSRGP_NLO' )

C
C$ Declarations_of_Input_and_Output_Arguments
C
      CHARACTER*3          IOFLAG
      CHARACTER*(*)        ITFLAG
      DOUBLE PRECISION     ITVALU
C
C$ Declarations_of_Local_Variables
C
      CHARACTER*3          IOUP
      CHARACTER*50         ITUP
C
C$ Save_Statements
C
      LOGICAL              NSET
      SAVE                 NSET

      LOGICAL              NHISET
      SAVE                 NHISET

      LOGICAL              NLOSET
      SAVE                 NLOSET

      DOUBLE PRECISION     NVAL
      SAVE                 NVAL

      DOUBLE PRECISION     NHIVAL
      SAVE                 NHIVAL

      DOUBLE PRECISION     NLOVAL
      SAVE                 NLOVAL
C
C$ Data_Statements
C
      DATA                 NSET       / .FALSE. /
      DATA                 NHISET     / .FALSE. /
      DATA                 NLOSET     / .FALSE. /
C
C$ Method
C-&

C      write(8,*) 'lsrgp:IOFLAG',IOFLAG
C      write(8,*) 'lsrgp:ITFLAG',ITFLAG

C1    Change input strings to upper case.

      IOUP = IOFLAG
      ITUP = ITFLAG

      CALL UPCASE( IOUP )
      CALL UPCASE( ITUP )

C1    Determine the action to be taken.

      IF ( IOUP .EQ. GET ) THEN

C2       You want to get a value...

         IF ( ITUP .EQ. NID ) THEN

            IF ( NSET ) THEN 
               ITVALU = NVAL
            ELSE
               WRITE(*,301) 
301            FORMAT(//,1X,'LSRGPB: No mean motion has been PUT yet.')
               STOP
            END IF
         ELSE IF ( ITUP .EQ. NIDHI ) THEN
            IF ( NHISET ) THEN 
               ITVALU = NHIVAL
            ELSE
               WRITE(*,302) 
302            FORMAT(//,1X,'LSRGPB: No NHI has been PUT yet.')
               STOP
            END IF
         ELSE IF ( ITUP .EQ. NIDLO ) THEN
            IF ( NLOSET ) THEN 
               ITVALU = NLOVAL
            ELSE
               WRITE(*,303) 
303            FORMAT(//,1X,'LSRGPB: No NLO has been PUT yet.')
               STOP
            END IF
         ELSE

C3          The item you want is not here.

            WRITE(*,397) ITFLAG
397         FORMAT(//,1X,'LSRGPB: ITFLAG(',A,') is not on the board.')

            STOP

         END IF

      ELSE IF ( IOUP .EQ. PUT ) THEN

C2       You want to put a value...

         IF ( ITUP .EQ. NID ) THEN

C3          You want to put the value of mean motion.

            NVAL = ITVALU
            NSET = .TRUE.
         ELSE IF ( ITUP .EQ. NIDHI ) THEN
            NHIVAL = ITVALU
            NHISET = .TRUE.
         ELSE IF ( ITUP .EQ. NIDLO ) THEN
            NLOVAL = ITVALU
            NLOSET = .TRUE.

         ELSE

C3          There is no place for your item.

            WRITE(*,398) ITFLAG
398         FORMAT(//,1X,'LSRGPB: There is no slot for ITFLAG(',A,').')

            STOP

         END IF

      ELSE

C2       You do not know what you want to do!

         WRITE(*,399) IOFLAG
399      FORMAT(//,1X,'LSRGPB: IOFLAG(',A3,') is not "GET" or "PUT".')

         STOP

      END IF

      RETURN
      END
