C$Procedure                         ORB2U
C
      SUBROUTINE ORB2U ( ORBIT , U )
C
C
C*******************************************************************************
C
C       Copyright (C) 1993, California Institute of Technology.  U.S.
C       Government Sponsorhip under NASA Contract NAS7-918 is
C       acknowledged.
C
C*******************************************************************************
C
C$ Log
C
C  10-JUL-1990 - Eric Cannell     - creation of ORB2U
C
C$ Purpose
C
C  ORB2U computes U, the argument of latitude of ORBIT.
C
C$ Input_Arguments
C
C  Name    Type   Dim        Units   Description
C  -----------------------------------------------------------------------------
C  ORBIT     DP     6       km,deg   classical orbital elements:
C                                    ORBIT(1) = semi-major axis
C                                    ORBIT(2) = eccentricity
C                                    ORBIT(3) = inclination
C                                    ORBIT(4) = longitude of the ascending node
C                                    ORBIT(5) = argument of periapsis
C                                    ORBIT(6) = mean anomaly
C
C$ Output_Arguments
C
C  Name    Type   Dim        Units   Description
C  -----------------------------------------------------------------------------
C  U         DP     1          deg   argument of latitude of ORBIT in 
C                                    range of 0..360.
C
C$ Restrictions
C
C  1] Note that MA2EA cannot transform mean anomalies > .965.
C
C$ Library_Links
C
C  TPXORB
C
C$ Parameters
C
      DOUBLE PRECISION     PI
      PARAMETER          ( PI     = 3. 14159 26535 89793 23846 D0 )

      DOUBLE PRECISION     D2R
      PARAMETER          ( D2R    = PI / 180D0 )

      DOUBLE PRECISION     TWOPI
      PARAMETER          ( TWOPI  = 2D0 * PI )
C
C$ Declarations_of_Input_and_Output_Arguments
C
      DOUBLE PRECISION     ORBIT  ( 6 )
      DOUBLE PRECISION     U
C
C$ Declarations_of_Local_Variables
C
      DOUBLE PRECISION     CE
      DOUBLE PRECISION     CV
      DOUBLE PRECISION     DENOM
      DOUBLE PRECISION     E
      DOUBLE PRECISION     EA
      DOUBLE PRECISION     SE
      DOUBLE PRECISION     SV
      DOUBLE PRECISION     V
C
C$ External_Statements
C
      DOUBLE PRECISION     MA2EA
      EXTERNAL             MA2EA

      DOUBLE PRECISION     RNG360
      EXTERNAL             RNG360
C                               
C$ Method
C-&

C1    Compute eccentric anomaly and its sine and cosine.

      E  = ORBIT(2)

      EA = MA2EA( E , ORBIT(6) ) * D2R
      SE = DSIN( EA )
      CE = DCOS( EA )

C1    Compute the common denominator.

      DENOM = 1D0 - E * CE

C1    Compute true anomaly.

      SV = DSQRT( 1D0 - E**2 ) * SE / DENOM
      CV = ( CE - E ) / DENOM

      V  = DATAN2( SV , CV ) / D2R

C1    Compute U, which is sum of argument of periapsis and true anomaly.

      U = RNG360( ORBIT(5) + V )

      RETURN
      END
